@@ -3,7 +3,7 @@ use strict;
 use Filter::Simple;
 
 use vars '$VERSION';
-$VERSION = '0.09';
+$VERSION = '0.10';
 
 =head1 NAME
 
@@ -13,7 +13,7 @@ Filter::signatures - very simplicistic signatures for Perl < 5.20
 
     use Filter::signatures;
     no warnings 'experimental::signatures'; # does not raise an error
-    use feature 'signatures'; # this now works on <5.16 as well
+    use feature 'signatures'; # this now works on <5.20 as well
 
     sub hello( $name ) {
         print "Hello $name\n";
@@ -26,12 +26,17 @@ Filter::signatures - very simplicistic signatures for Perl < 5.20
     }
     hello2(); # Hello world
 
+=head1 DESCRIPTION
+
+This module implements a backwards compatibility shim for formal Perl subroutine
+signatures that were introduced to the Perl core with Perl 5.20.
 
 =head1 CAVEATS
 
-This implements a very simplicistic transform to allow for using very
+The technique used is a very simplicistic transform to allow for using very
 simplicistic named formal arguments in subroutine declarations. This module
-does not implement warning if more parameters than expected are passed in.
+does not implement warning if more or fewer parameters than expected are
+passed in.
 
 The module also implements default values for unnamed parameters by
 splitting the formal parameters on C<< /,/ >> and assigning the values
@@ -57,9 +62,31 @@ A better hotfix is to upgrade to Perl 5.20 or higher and use the native
 signatures support there. No other code change is needed, as this module will
 disable its functionality when it is run on a Perl supporting signatures.
 
+=head2 Line Numbers
+
+Due to a peculiarity of how Filter::Simple treats here documents in some
+versions, line numbers may get out of sync if you use here documents.
+
+If you spread your formal signatures across multiple lines, the line numbers
+may also go out of sync with the original document.
+
+=head2 Comments within signatures
+
+The module does not support comments within signatures
+
+  sub invalid (
+      $name,     # use this as the name
+      $location, # use this as the location
+  ) {
+      "This is an example"
+  }
+
+The workaround is to not do that or to upgrade to Perl 5.20 or higher
+and use the native signatures support there.
+
 =head2 C<< eval >>
 
-It seems that L<Filter::Simple> does not trigger when using
+L<Filter::Simple> does not trigger when using
 code such as
 
   eval <<'PERL';
@@ -73,6 +100,11 @@ code such as
 So, creating subroutines with signatures from strings won't work with
 this module. The workaround is to upgrade to Perl 5.20 or higher.
 
+=head2 Deparsing
+
+The generated code does not deparse identically to the code generated on a
+Perl with native support for signatures.
+
 =head1 ENVIRONMENT
 
 If you want to force the use of this module even under versions of
@@ -89,13 +121,20 @@ my $have_signatures = eval {
 };
 
 sub parse_argument_list {
-    my( $name, $arglist ) = @_;
+    my( $name, $arglist, $whitespace ) = @_;
     (my $args=$arglist) =~ s!^\((.*)\)!$1!;
     my @args = split /\s*,\s*/, $args; # a most simple argument parser
     my $res;
+    # Adjust how man newlines we gobble
+    $whitespace ||= '';
+    #warn "[[$whitespace$args]]";
+    my $padding = () = (($whitespace . $args) =~ /\n/smg);
     if( @args ) {
         my @defaults;
         for( 0..$#args ) {
+            # Keep everything on one line
+            $args[$_] =~ s/\n/ /g;
+
             # Named argument
             if( $args[$_] =~ /^\s*([\$\%\@]\s*\w+)\s*=/ ) {
                 my $named = "$1";
@@ -111,7 +150,9 @@ sub parse_argument_list {
                 $args[$_] = 'undef';
             }
         };
-        $res = sprintf 'sub %s { my (%s)=@_;%s', $name, join(",", @args), join( "" , @defaults);
+        $res = sprintf 'sub %s { my (%s)=@_;%s%s', $name, join(",", @args), join( "" , @defaults), "\n" x $padding;
+        # die sprintf("Too many arguments for subroutine at %s line %d.\n", (caller)[1, 2]) unless @_ <= 2
+        # die sprintf("Too few arguments for subroutine at %s line %d.\n", (caller)[1, 2]) unless @_ >= 2
     } else {
         $res = sprintf 'sub %s { @_==0 or warn "Subroutine %s called with parameters.";', $name, $name;
     };
@@ -124,10 +165,10 @@ sub transform_arguments {
 	# sub foo($x,$y,@) { ... }, throwing away additional arguments
 	# Named or anonymous subs
 	no warnings 'uninitialized';
-	s{\bsub\s*(\w*)\s*\(((?:[^)]*?\@?))\)\s*\{}{
-		parse_argument_list("$1","$2")
+	s{\bsub(\s*)(\w*)(\s*)\((\s*)((?:[^)]*?\@?))(\s*)\)(\s*)\{}{
+		parse_argument_list("$2","$5","$1$3$4$6$7")
 	 }mge;
-	 $_
+	$_
 }
 
 if( (! $have_signatures) or $ENV{FORCE_FILTER_SIGNATURES} ) {
